import Flutter
import UIKit
import PhotoEditorSDK



@available(iOS 9.0, *)
public class SwiftPhotoEditorSdkPlugin: NSObject, FlutterPlugin {

    var controller: UIViewController!
    var globalResult: FlutterResult!
    let podBundle = Bundle(for: SwiftPhotoEditorSdkPlugin.self) // for getting pod url

    public static func register(with registrar: FlutterPluginRegistrar) {
        self.translate()
        if let licenseURL = Bundle.main.url(forResource: "license", withExtension: "") {
            PESDK.unlockWithLicense(at: licenseURL)
        }
        let channel = FlutterMethodChannel(name: "photo_editor_sdk", binaryMessenger: registrar.messenger())
        let instance = SwiftPhotoEditorSdkPlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
    }

    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        controller = UIApplication.shared.keyWindow?.rootViewController
        globalResult = result
        let methodIdentifier = MethodIdentifierEnum(rawValue: call.method)
        switch(methodIdentifier) {
            case .addAllContents:
                if let args = call.arguments as? Dictionary<String, Array<String>> {
                    UserDefaults.standard.set(args["logos"], forKey: "logos")
                    UserDefaults.standard.set(args["stickers"], forKey: "stickers")
                    setUpStickers()
                }
                break
            case .editImageByUrl:
                guard let args = call.arguments as? String,
                      let url = URL(string: args)
                else {
                      return
                }

                do {
                    let data = try Data(contentsOf: url)
                    if let uiImage = UIImage(data: data) {
                        let photoImage = Photo(image: uiImage)
                        self.presentPhotoEditViewController(photo: photoImage)
                    }
                } catch {
                    print(error)
                }
                break
            case .editImageByFile:
                let args = call.arguments as! String
                let image : Photo = Photo(image: UIImage(named: args)!)
                self.presentPhotoEditViewController(photo: image)
                break
            default:
                break
        }
    }

    private func presentPhotoEditViewController(photo: Photo) {
        DispatchQueue.main.async {
            self.controller.present(self.createPhotoEditViewController(with: photo), animated: true, completion: nil)
        }
    }

    private func createPhotoEditViewController(with photo: Photo, and photoEditModel: PhotoEditModel = PhotoEditModel()) -> PhotoEditViewController {
        let configuration = buildConfiguration()

        // Create a photo edit view controller
        let photoEditViewController = PhotoEditViewController(photoAsset: photo, configuration: configuration, photoEditModel: photoEditModel)

        photoEditViewController.modalPresentationStyle = .fullScreen
        photoEditViewController.delegate = self

        return photoEditViewController
    }

    func createDefaultItems() -> [PhotoEditMenuItem] {
        guard let imageLogo = UIImage(named: "ico_logo", in: podBundle, compatibleWith: nil),
            let imageStickers = UIImage(named: "ico_sticker", in: podBundle, compatibleWith: nil),
            let imageText = UIImage(named: "ico_text", in: podBundle, compatibleWith: nil) else {
            return []
        }

        let tool = ToolMenuItem(
            title: "logos",
            icon: imageLogo,
            toolControllerClass: LogoStickerToolController.self)!

        let logos = PhotoEditMenuItem.tool(tool)

        let adesivos = PhotoEditMenuItem.tool(
            ToolMenuItem(
                title: "adesivos",
                icon: imageStickers,
                toolControllerClass: StickerToolController.self)!
        )

        let text = PhotoEditMenuItem.tool(ToolMenuItem(title: "texto", icon: imageText, toolControllerClass: TextToolController.self)!)

        //Add logo when ready
        return [adesivos, text]
    }

    private func buildConfiguration() -> Configuration {
        let configuration = Configuration { builder in
            // Configure editor
            builder.configurePhotoEditViewController { options in
                options.menuItems = createDefaultItems()

                options.titleViewConfigurationClosure = { view in
                    /// This will add and UILabel upside of title of the toolBar. Creating a layout more trustworthy with Ahazou.
                    let label = self.createTitleToolbarLabel(title: "editor")
                    view.addSubview(label)
                    self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
                }

                options.discardButtonConfigurationClosure = { button in
                    button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
                }

                options.applyButtonConfigurationClosure = { button in
                    /// Editing the download/save button of PhotoEditView
                    button.setImage(UIImage(), for: .normal)
                    button.setTitle("salvar", for: .normal)
                    button.titleLabel?.font = UIFont.systemFont(ofSize: 18, weight: .semibold)
                    button.titleEdgeInsets.right = 20
                    button.titleEdgeInsets.left = -20
                }
            }

            builder.configureStickerToolController { (options) in
                options.discardButtonConfigurationClosure = { button in
                    button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
                }

                options.titleViewConfigurationClosure = { view in
                    let label = self.createTitleToolbarLabel(title: "adesivos")
                    view.addSubview(label)
                    self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
                }
            }
            
            builder.configureTextToolController { (options) in
                options.discardButtonConfigurationClosure = { button in
                    button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
                }

                options.titleViewConfigurationClosure = { view in
                    let label = self.createTitleToolbarLabel(title: "editor")
                    view.addSubview(label)
                    self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
                }
            }
            
            builder.configureStickerOptionsToolController { (options) in
                options.discardButtonConfigurationClosure = { button in
                    button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
                }

                options.titleViewConfigurationClosure = { view in
                    let label = self.createTitleToolbarLabel(title: "opções")
                    view.addSubview(label)
                    self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
                }
            }
            
            builder.configureTextFontToolController { (options) in
                options.discardButtonConfigurationClosure = { button in
                    button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
                }

                options.titleViewConfigurationClosure = { view in
                    let label = self.createTitleToolbarLabel(title: "opções")
                    view.addSubview(label)
                    self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
                }
            }
            
            builder.configureTextColorToolController { (options) in
                options.discardButtonConfigurationClosure = { button in
                    button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
                }

                options.titleViewConfigurationClosure = { view in
                    let label = self.createTitleToolbarLabel(title: "opções")
                    view.addSubview(label)
                    
                    self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
                }
            }
            
            builder.configureTextOptionsToolController { (options) in
                options.discardButtonConfigurationClosure = { button in
                    button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
                }

                options.titleViewConfigurationClosure = { view in
                    let label = self.createTitleToolbarLabel(title: "opções")
                    view.addSubview(label)
                    self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
                }
            }

            // Configure theme
            builder.theme = self.theme
        }

        return configuration
    }
    
    fileprivate func createTitleToolbarLabel(title: String) -> UILabel {
        let padding = UIEdgeInsets(top: 0, left: 10, bottom: 0, right: 0)   // top-left-bottom-right
        /// The right and left button has 60 width. 60 + 60 = 120 but the "save" button has a -20 left padding so 120 + 20 = 140
        let rect = CGRect(x: 0, y: 0, width: UIScreen.main.bounds.width - 140, height: 44).inset(by: padding)
        let label = UILabel(frame: rect)
        label.textAlignment = .left
        label.text = title
        label.textColor = .black
        label.backgroundColor = .white


        label.font = UIFont.systemFont(ofSize: 28, weight: .bold)
        
        return label
    }
    
    fileprivate func applyConstraintsOnTitleToolbarLabel(label: UILabel, view: UIView) {
        label.topAnchor.constraint(equalTo: view.topAnchor, constant: 0).isActive = true
        label.bottomAnchor.constraint(equalTo: view.bottomAnchor, constant: 0).isActive = true
        label.rightAnchor.constraint(equalTo: view.rightAnchor, constant: 0).isActive = true
        label.leftAnchor.constraint(equalTo: view.leftAnchor, constant: 0).isActive = true
    }

    static func preferredLocale() -> Locale {
        guard let preferredIdentifier = Locale.preferredLanguages.first else {
            return Locale.current
        }
        return Locale(identifier: preferredIdentifier)
    }
    
    fileprivate static func translate() {
        PESDK.localizationBlock = { stringToLocalize in
            return NSLocalizedString(stringToLocalize, tableName: "MyStrings", bundle: Bundle(for: SwiftPhotoEditorSdkPlugin.self), comment: "")
        }
    }
    
    fileprivate func setUpStickers() {
        StickerCategory.all.removeAll()
        if let last = StickerCategory.all.last {
            let shapes = StickerCategory(title: "Formas", imageURL: last.imageURL, stickers: last.stickers)
            StickerCategory.all.append(shapes)
        }
        createStickerSection(key: "logos", name: "Logos", index: 1)
        createStickerSection(key: "stickers", name: "Adesivos", index: 2)
    }

    private func createStickerSection(key: String, name: String, index: Int) {

        var images: [Sticker] = []
        let defaults = UserDefaults.standard
        let logos = defaults.array(forKey: key)
        var cont = 0
        logos?.forEach { logo in
            if let logoURL = URL(string: logo as! String) {
                let logoSticker = Sticker(imageURL: logoURL, thumbnailURL: nil, identifier: "\(cont) \(index)")
                cont = cont+1
                images.append(logoSticker)
            }
        }

        if !images.isEmpty {
            if let last = images.last {
                let stickerCategory = StickerCategory(title: name, imageURL: last.imageURL, stickers: images)

                StickerCategory.all.insert(stickerCategory, at: StickerCategory.all.count)
            }
        }
    }

    private static let defaultTheme: Theme = {
        return Theme(backgroundColor: Colors.neutral50, menuBackgroundColor: .white, toolbarBackgroundColor: .white, primaryColor: Colors.primmary)
    }()

    private var theme = defaultTheme
}

@available(iOS 9.0, *)
extension SwiftPhotoEditorSdkPlugin: PhotoEditViewControllerDelegate {
    public func photoEditViewController(_ photoEditViewController: PhotoEditViewController, didSave image: UIImage, and data: Data) {
        if let navigationController = photoEditViewController.navigationController {
            navigationController.popViewController(animated: true)
        } else {
            globalResult(data)

            controller.dismiss(animated: true, completion: nil)
        }
    }

    public func photoEditViewControllerDidFailToGeneratePhoto(_ photoEditViewController: PhotoEditViewController) {
        if let navigationController = photoEditViewController.navigationController {
            navigationController.popViewController(animated: true)
        } else {
            controller.dismiss(animated: true, completion: nil)
        }
    }

    public func photoEditViewControllerDidCancel(_ photoEditViewController: PhotoEditViewController) {
        if let navigationController = photoEditViewController.navigationController {
            navigationController.popViewController(animated: true)
        } else {
            controller.dismiss(animated: true, completion: nil)
        }
    }
}
